@@ -15,10 +15,7 @@
 #    under the License.
 
 import argparse
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
+from collections import OrderedDict
 import hashlib
 import logging
 import os
@@ -27,8 +24,8 @@ import traceback
 import uuid
 
 import fixtures
-from keystoneclient import exceptions as ks_exc
-from keystoneclient import fixture as ks_fixture
+from keystoneauth1 import exceptions as ks_exc
+from keystoneauth1 import fixture as ks_fixture
 import mock
 from requests_mock.contrib import fixture as rm_fixture
 import six
@@ -163,6 +160,35 @@ class ShellTest(testutils.TestCase):
             sys.stderr = orig_stderr
         return (stdout, stderr)
 
+    def test_fixup_subcommand(self):
+        arglist = [u'image-list', u'--help']
+        if six.PY2:
+            expected_arglist = [b'image-list', u'--help']
+        elif six.PY3:
+            expected_arglist = [u'image-list', u'--help']
+
+        openstack_shell.OpenStackImagesShell._fixup_subcommand(
+            arglist, arglist
+        )
+        self.assertEqual(expected_arglist, arglist)
+
+    def test_fixup_subcommand_with_options_preceding(self):
+        arglist = [u'--os-auth-token', u'abcdef', u'image-list', u'--help']
+        unknown = arglist[2:]
+        if six.PY2:
+            expected_arglist = [
+                u'--os-auth-token', u'abcdef', b'image-list', u'--help'
+            ]
+        elif six.PY3:
+            expected_arglist = [
+                u'--os-auth-token', u'abcdef', u'image-list', u'--help'
+            ]
+
+        openstack_shell.OpenStackImagesShell._fixup_subcommand(
+            unknown, arglist
+        )
+        self.assertEqual(expected_arglist, arglist)
+
     def test_help_unknown_command(self):
         shell = openstack_shell.OpenStackImagesShell()
         argstr = '--os-image-api-version 2 help foofoo'
@@ -250,7 +276,9 @@ class ShellTest(testutils.TestCase):
 
     def test_get_base_parser(self):
         test_shell = openstack_shell.OpenStackImagesShell()
-        actual_parser = test_shell.get_base_parser()
+        # NOTE(stevemar): Use the current sys.argv for base_parser since it
+        # doesn't matter for this test, it just needs to initialize the CLI
+        actual_parser = test_shell.get_base_parser(sys.argv)
         description = 'Command-line interface to the OpenStack Images API.'
         expected = argparse.ArgumentParser(
             prog='glance', usage=None,
@@ -276,17 +304,6 @@ class ShellTest(testutils.TestCase):
         self.assertEqual('mycert', args.os_cert)
         self.assertEqual('mykey', args.os_key)
 
-        # make sure we get the same thing with --cert-file and --key-file
-        args = ('--os-image-api-version 2 '
-                '--cert-file mycertfile '
-                '--key-file mykeyfile image-list')
-        glance_shell = openstack_shell.OpenStackImagesShell()
-        glance_shell.main(args.split())
-        assert mock_versioned_client.called
-        ((api_version, args), kwargs) = mock_versioned_client.call_args
-        self.assertEqual('mycertfile', args.os_cert)
-        self.assertEqual('mykeyfile', args.os_key)
-
     @mock.patch('glanceclient.v1.client.Client')
     def test_no_auth_with_token_and_image_url_with_v1(self, v1_client):
         # test no authentication is required if both token and endpoint url
@@ -637,7 +654,7 @@ class ShellTestWithKeystoneV3Auth(ShellTest):
         glance_shell.main(args.split())
         self.assertEqual(0, self.v3_auth.call_count)
 
-    @mock.patch('keystoneclient.discover.Discover',
+    @mock.patch('keystoneauth1.discover.Discover',
                 side_effect=ks_exc.ClientException())
     def test_api_discovery_failed_with_unversioned_auth_url(self,
                                                             discover):
