@@ -240,22 +240,30 @@ class Controller(object):
         return body, resp
 
     @utils.add_req_id_to_object()
-    def stage(self, image_id, image_data):
+    def stage(self, image_id, image_data, image_size=None):
         """Upload the data to image staging.
 
         :param image_id: ID of the image to upload data for.
         :param image_data: File-like object supplying the data to upload.
+        :param image_size: Unused - present for backwards compatibility
         """
         url = '/v2/images/%s/stage' % image_id
-        return self.upload(image_id,
-                           image_data,
-                           u_url=url)
+        resp, body = self.upload(image_id,
+                                 image_data,
+                                 u_url=url)
+        return body, resp
 
     @utils.add_req_id_to_object()
-    def image_import(self, image_id, method='glance-direct'):
+    def image_import(self, image_id, method='glance-direct', uri=None):
         """Import Image via method."""
         url = '/v2/images/%s/import' % image_id
-        data = {'import_method': method}
+        data = {'method': {'name': method}}
+        if uri:
+            if method == 'web-download':
+                data['method']['uri'] = uri
+            else:
+                raise exc.HTTPBadRequest('URI is only supported with method: '
+                                         '"web-download"')
         resp, body = self.http_client.post(url, data=data)
         return body, resp
 
